import scriptconfig as scfg


class Config(scfg.Config):
    """
    Default configuration for meta-baseline
    """
    default = {
        'name': scfg.Value('meta-baseline',
                help='name of the proposed method'),

        'cls_transform': scfg.Value('identity',
                help='data transform used in classification pre-training'),
        'cls_batch_size': scfg.Value(256,
                help='batch size in classification pre-training'),
        'cls_max_epoch': scfg.Value(4,
                help='max epoch in classification pre-training'),
        'cls_lr': scfg.Value(0.01,
                help='lr in classification pre-training'),
        'cls_milestones': scfg.Value([2],
                help='epochs with lr decay in classification pre-training'),

        'meta_transform': scfg.Value('identity',
                help='data transform used in meta-learning stage'),
        'meta_batch_size': scfg.Value(4,
                help='batch size in meta-learning stage'),
        'meta_epoch_size': scfg.Value(10,
                help='num of batches in each epoch in meta-learning stage'),
        'meta_lr': scfg.Value(0.001,
                help='lr in meta-learning stage'),
        'meta_max_epoch': scfg.Value(2,
                help='max epoch in meta-learning stage'),
        'meta_freeze_bn': scfg.Value(False,
                help='freezing the BN layers in meta-learning stage'),

        'eval_batch_size': scfg.Value(256,
                help='batch size in evaluation'),

        'cuda': scfg.Value(True,
                help='whether or not to use cuda'),

        'gpu_ids': scfg.Value('0,',
                help='GPUs to use (in format 0, or 0,1,2...'),
        'num_workers': scfg.Value(8,
                help='number of workers in the dataloader'),
    }
